# Reference
<!-- DO NOT EDIT: This document was generated by Puppet Strings -->

## Table of Contents

**Classes**

* [`role`](#role): Assigns the correct role to each node.  It acts as a proxy which allows you to store your roles in a different namespaced module. For a singl
* [`role::default`](#roledefault): Dummy default role that does nothing.

**Functions**

* [`role::expand_search_namespaces`](#roleexpand_search_namespaces): Sanitizes Role::SearchNamespaces for use in `role`.
* [`role::translate_double_underscores`](#roletranslate_double_underscores): Translates all double (or more) underscores in a role to '::'
* [`role::translate_slash`](#roletranslate_slash): Translates all (repeated) slashes in a role to '::'
* [`role::translate_with_map`](#roletranslate_with_map): Translate a role by executing a gsubstr(pattern, value, 'G') with patterns and values from the provided map.

**Data types**

* [`Role::ResolveMethod`](#roleresolvemethod): Supported resolve methods
* [`Role::SearchNamespace`](#rolesearchnamespace): A SearchNamespace can be a single string or a hash.

## Classes

### role

Assigns the correct role to each node.

It acts as a proxy which allows you to store your roles in a different namespaced
module. For a single host, you can also use a specific configuration (upstream / shared
roles and profiles).

To get started, you should define the namespace (or search_namespaces) to use.

#### Parameters

The following parameters are available in the `role` class.

##### `namespace`

Data type: `Optional[String]`

The module namespace that holds your roles. If you are not using namespaces and map
roles directly to class names, make this an empty string ('').

Default value: `undef`

##### `search_namespaces`

Data type: `Optional[Array[Role::SearchNamespace]]`

A list of namespaces to search. If using this, we will attempt to find the
first existing role in order. If no match is found, the puppet run will fail.

Default value: `undef`

##### `separator`

Data type: `String`

Anything to put in between the namespace and the role.

Default value: '::'

##### `resolve_order`

Data type: `Variant[Role::ResolveMethod, Array[Role::ResolveMethod]]`

The order in which we will be looking for the correct
role to use. Currently supported values are:
* `trusted`: Use a trusted fact. The name must be configured using `trusted_extension_name`.
* `fact`: Use a fact. The name must be configured using `fact_name`.
* `param`: Uses the provided `role` parameter. This can also be used to configure using hiera.
* `callback`: Use a function callback. The function must be configured using `function_callback_name`. (Not available on puppet 4.x!)
* `default`: Fall back to the default value. You would typically put this after other methods.
* `fail`: Fail the run if this method is reached. This enforces setting up a role and skips using the default role.

Default value: ['param', 'default']

##### `role`

Data type: `Optional[String[1]]`

The role this node should get.

Default value: `undef`

##### `trusted_extension_name`

Data type: `Optional[String[1]]`

Name of the trusted fact (extension).

Default value: `undef`

##### `fact_name`

Data type: `Optional[String[1]]`

Name of the fact that contains the role.

Default value: `undef`

##### `function_callback_name`

Data type: `Optional[String[1]]`

A function that returns the role.

Default value: `undef`

##### `translate_role_callback`

Data type: `Variant[Undef, String[1], Hash]`

Optionally, a function name that should be used or a map with gsubstr tuples.
* function name: A puppet function to call.
    It should accept a single value and return a string. (Not available on puppet 4.x!)
    See `role::translate_slash` and `role::translate_double_underscores` for examples.
* a Hash: A mapping with keypairs that is passed to `role::translate_with_map`.

Default value: `undef`

##### `default_role`

Data type: `String`

the default role to assume. Used when no resolve method provides a result.

Default value: 'default'

##### `default_namespace`

Data type: `String`

namespace to use if the default is used.

Default value: '::role'

##### `default_separator`

Data type: `String`

separator to use if the default is used.

Default value: '::'

### role::default

Dummy default role that does nothing.

## Functions

### role::expand_search_namespaces

Type: Puppet Language

This function is used to sanitize user input and return a single map with namespace - separator entries.

#### Examples

##### hiera configured namespaces

```puppet

$search_namespaces = [
  '',
  { 'my_roles' => '_' },
  'public_roles',
]
$expanded = role::expand_search_namespaces('::', $search_namespaces)
# => {'' => '::', 'my_roles' => '_', 'public_roles' => '::' }
```

#### `role::expand_search_namespaces(String $separator, Variant[Role::SearchNamespace, Array[Role::SearchNamespace]] $search)`

This function is used to sanitize user input and return a single map with namespace - separator entries.

Returns: `Hash[String, String]` Expanded configuration.

##### Examples

###### hiera configured namespaces

```puppet

$search_namespaces = [
  '',
  { 'my_roles' => '_' },
  'public_roles',
]
$expanded = role::expand_search_namespaces('::', $search_namespaces)
# => {'' => '::', 'my_roles' => '_', 'public_roles' => '::' }
```

##### `separator`

Data type: `String`

Default separator to use when the namespace does not provide one.

##### `search`

Data type: `Variant[Role::SearchNamespace, Array[Role::SearchNamespace]]`

`Role::SearchNamespace`s to expand.

### role::translate_double_underscores

Type: Puppet Language

Translates all double (or more) underscores in a role to '::'

#### `role::translate_double_underscores(String $role)`

Translates all double (or more) underscores in a role to '::'

Returns: `String` Translated role.

##### `role`

Data type: `String`

Role to perform translate on.

### role::translate_slash

Type: Puppet Language

Translates all (repeated) slashes in a role to '::'

#### `role::translate_slash(String $role)`

Translates all (repeated) slashes in a role to '::'

Returns: `String` Translated role.

##### `role`

Data type: `String`

Role to perform translate on.

### role::translate_with_map

Type: Puppet Language

Translate a role by executing a gsubstr(pattern, value, 'G') with patterns and
values from the provided map.

#### `role::translate_with_map(String $role, Hash[
    Variant[String, Regexp],
    String
  ] $map, Boolean $strict = false)`

Translate a role by executing a gsubstr(pattern, value, 'G') with patterns and
values from the provided map.

Returns: `String` Translated role.

##### `role`

Data type: `String`

Role to perform translate on.

##### `map`

Data type: `Hash[
    Variant[String, Regexp],
    String
  ]`

A map with gsubstr translations with key as pattern and value
as replacement.

##### `strict`

Data type: `Boolean`

Experimental parameter. If enabled, fails the puppet run when
a role includes characters that are in the target map present as values.
This would indicate that a role is being provided which would not
have expected replacement values in use. (This could happen during a migration,
for example.

## Data types

### Role::ResolveMethod

Supported resolve methods

Alias of `Enum['trusted', 'param', 'fact', 'callback', 'default', 'fail']`

### Role::SearchNamespace

A SearchNamespace can be a single string or a hash.

Alias of `Variant[String, Hash[String, Optional[String]]]`

